/***********************************************************************
 *
 * Copyright (C) 2016-2020 wereturtle
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#ifndef PREFERENCESDIALOG_H
#define PREFERENCESDIALOG_H

#include <QDialog>
#include <QObject>
#include <QScopedPointer>

namespace ghostwriter
{
/**
 * Displays application preferences.
 */
class PreferencesDialogPrivate;
class PreferencesDialog : public QDialog
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(PreferencesDialog)

public:
    /**
     * Constructor.
     */
    PreferencesDialog(QWidget *parent = nullptr);

    /**
     * Destructor.
     */
    virtual ~PreferencesDialog();

private:
    QScopedPointer<PreferencesDialogPrivate> d_ptr;
};
} // namespace ghostwriter

#endif // PREFERENCESDIALOG_H
